##### MNL model for body type count #####

##### Description #####
# This code estimates and applies an MNL model of body type count on the same data set 


library("utils")   # to use csv read function
library("foreign") # to use spss read function
library("pastecs") # to use descriptive statistics function
library("mlogit")
library("graphics")
library("VGAM")
library("ZeligChoice") # This package is for actual mlogit function
library("aod")
library("plotrix")


pst <- proc.time()


##### Read the data #####
data <- read.table("C:/workspace/VFC_Package_Beta/Model_Estimation/az_hhld_vfc_cleaned_final_restructured.dat", 
col.names = c("HOUSEID",
              "TAZ",
              "RAZ",
              "uno",
              "sero",
              "id",
              "CAR1",
              "CAR2",
              "CAR3",
              "VAN1",
              "VAN2",
              "VAN3",
              "SUV1",
              "SUV2",
              "SUV3",
              "PICK1",
              "PICK2",
              "PICK3",
              "MOTOR",
              "DCAR1",
              "DCAR2",
              "DCAR3",
              "DVAN1",
              "DVAN2",
              "DVAN3",
              "DSUV1",
              "DSUV2",
              "DSUV3",
              "DPICK1",
              "DPICK2",
              "DPICK3",
              "DMOTER",
              "PUMA",
              "DISTRICT",
              "DRVRCNT",
              "HH_HISP",
              "HH_RACE",
              "HHBG",
              "HHC_MSA",
              "HHFAMINC",
              "HHMETDIV",
              "HHRELATD",
              "HHRESP",
              "HHSIZE",
              "HHVEHCNT",
              "HOMEGEO",
              "HOMELAT",
              "HOMELONG",
              "HOMEOWN",
              "HOMETYPE",
              "IMPTHOWN",
              "NUMADLT",
              "RAIL",
              "RESP_CNT",
              "TRAVDAY",
              "URBAN",
              "URBANSIZE",
              "URBRUR",
              "WRKCOUNT",
              "HH13WGT",
              "WTHHFIN",
              "HHR_HISP",
              "HHR_RACE",
              "LIF_CYC",
              "CNTTDHH",
              "num_pers",
              "num_trips",
              "TRPCNT",
              "VEHCNT",
              "YEAR",
              "POP1",
              "POP2",
              "POP3",
              "POP4",
              "HH1",
              "HH2",
              "HH3",
              "HH4",
              "EMP1",
              "EMP2",
              "EMP3",
              "EMP4",
              "EMP5",
              "INC1",
              "INC2",
              "INC3",
              "INC4",
              "INC5",
              "TOTAREA",
              "OFFAREA",
              "ENROLL",
              "RFLAG",
              "APASS",
              "DU9",
              "DU19",
              "DU29",
              "DU30",
              "SFHH",
              "MFHH",
              "POPCORR",
              "POPINST",
              "POPMINST",
              "WAHEMP",
              "CONSTEMP",
              "PUBOFF",
              "PUBPUB",
              "ASUENROLL",
              "K8",
              "G9TO12",
              "NSBEMP",
              "POPDEN",
              "EMPDEN",
              "OWN",
              "URB",
              "RUR",
              "SF",
              "MF",
              "INCOME1",
              "INCOME2",
              "INCOME3",
              "INCOME4",
              "INCOME5",
              "NUMCHILD",
              "child_pres",
              "WORK0",
              "WORK1",
              "WORK2",
              "WORK3",
              "HSIZE1",
              "HSIZE2",
              "HSIZE3",
              "HSIZE4",
              "annual_bike_mile_sum",
              "annual_walk_mile_sum",
              "trp_bike_walk_ind",
              "per_bike_ind",
              "per_walk_ind",
              "ann_mile_ind",
              "annual_non_motorized_mileage",
              "SF_Own",
              "Retired",
              "popden_per_100",
              "empden_per_100",
              "TOT_POP",
              "TOT_EMP",
              "TOTAL_HH",
              "Prop_HH_INC5",
              "Prop_HH_INC1",
              "Prop_SFHH",
              "Prop_MFHH",
              "annual_bw_trp_miles_hhld_PB",
              "annual_bw_trp_miles_hhld_raw",
              "trip_bike_walk_ind",
              "per_bike_walk_ind",
              "ind_non_motor_mileage",
              "annual_non_motor_mileage_new",
              "test",
              "TRANSIT20",
              "TRANSIT50",
              "AUTO20",
              "AUTO50",
              "RATE_TRANSIT20",
              "RATE_TRANSIT50",
              "RATE_AUTO20",
              "RATE_AUTO50",
              "CURRENT",
              "FUTURE",
              "Dev_area",
              "TAZ_density",
              "TRANSIT10",
              "TRANSIT30",
              "AUTO10",
              "AUTO30",
              "RATE_TRANSIT10",
              "RATE_TRANSIT30",
              "RATE_AUTO10",
              "RATE_AUTO30",
              "TAZ_den_Q4",
              "TAZ_den_Q3",
              "TAZ_den_Q2",
              "TAZ_den_Q1",
              "AUTO10_Q4",
              "AUTO10_Q3",
              "AUTO10_Q2",
              "AUTO10_Q1",
              "AUTO30_Q4",
              "AUTO30_Q3",
              "AUTO30_Q2",
              "AUTO30_Q1",
              "CAR0_3",
              "CAR4_9",
              "CAR_10",
              "VAN0_3",
              "VAN4_9",
              "VAN_10",
              "SUV0_3",
              "SUV4_9",
              "SUV_10",
              "PICK0_3",
              "PICK4_9",
              "PICK_10",
              "TAZ_den_Q4_Old",
              "TAZ_den_Q3_Old",
              "TAZ_den_Q2_Old",
              "TAZ_den_Q1_Old",
              "AUTO10_Q4_Old",
              "AUTO10_Q3_Old",
              "AUTO10_Q2_Old",
              "AUTO10_Q1_Old",
              "AUTO30_Q4_Old",
              "AUTO30_Q3_Old",
              "AUTO30_Q2_Old",
              "AUTO30_Q1_Old",
              "carmile",
              "vanmile",
              "suvmile",
              "pickmile",
              "NCAR",
              "NVAN",
              "NSUV",
              "NPICK",
              "NMOTOR",
              "YCAR0_5",
              "YCAR6_11",
              "YCAR12",
              "YVAN0_5",
              "YVAN6_11",
              "YVAN12",
              "YSUV0_5",
              "YSUV6_11",
              "YSUV12",
              "YPICK0_5",
              "YPICK6_11",
              "YPICK12",
              "NTYPE",
              "Index1",
              "CTYPE1"),na.strings="-999")

##### Descriptive Statistics #####
# stat.desc(data,basic=TRUE, desc=TRUE, norm=FALSE, p=0.95); 


##### Coefficient Definitions for MNL Model#####
data$const <- 1

data$inc1_0 <- data$INCOME1 * data$const 
data$inc2_0 <- data$INCOME2 * data$const
data$own_0 <- data$OWN * data$const
data$hsiz1_0 <- data$HSIZE1 * data$const
data$wrkr0_0 <- data$WORK0 * data$const
data$popden_0 <- data$POPDEN * data$const

data$inc1_1 <- data$INCOME1 * data$const
data$inc2_1 <- data$INCOME2 * data$const
data$hsiz1_1 <- data$HSIZE1 * data$const
data$promf_1 <- data$Prop_MFHH * data$const
data$chldpr_1 <- data$child_pres * data$const

data$rur_2 <- data$RUR * data$const
data$inc4_2 <- data$INCOME4 * data$const
data$inc5_2 <- data$INCOME5 * data$const
data$hsiz4_2 <- data$HSIZE4 * data$const
data$own_2 <- data$OWN * data$const

data$own_3 <- data$OWN * data$const
data$nadlt_3 <- data$NUMADLT * data$const
data$wrkr3_3 <- data$WORK3 * data$const
data$popden_3 <- data$POPDEN * data$const
data$chldpr_3 <- data$child_pres * data$const


#### Multinomial logit model estimation #####

body_type_count <- zelig(list(id(Index1, 1) ~ inc1_0 + inc2_0 + own_0 + hsiz1_0 + wrkr0_0 + popden_0,
                                 id(Index1, 2) ~ inc1_1 + inc2_1 + hsiz1_1 + promf_1 + chldpr_1, 
                                 id(Index1, 3) ~ rur_2 + inc4_2 + inc5_2 + hsiz4_2 + own_2,
                                 id(Index1, 4) ~ 0 + own_3 + nadlt_3 + wrkr3_3 + popden_3 + chldpr_3), model = "mlogit", data = data)
summary(body_type_count)


######################################### Application ###############################################
num_body <- 4
coefs <-coef(body_type_count)
cons <- coefs[1:num_body-1]
zero_coef <- coefs[4:9]
one_coef <- coefs[10:14]
two_coef <- coefs[15:19]
three_coef <- coefs[20:24]
four_coef <- data$INCOME1*0


### Utility computation
util_zero_body <- exp(cons[1] + data$INCOME1*zero_coef[1] + data$INCOME2*zero_coef[2] + data$OWN*zero_coef[3]+ data$HSIZE1*zero_coef[4] + data$WORK0*zero_coef[5]
                      + data$POPDEN*zero_coef[6])
util_one_body <- exp(cons[2] + data$INCOME1*one_coef[1] + data$INCOME2*one_coef[2] + data$HSIZE1*one_coef[3]+ data$Prop_MFHH*one_coef[4] + data$child_pres*one_coef[5])
util_two_body <- exp(cons[3] + data$RUR*two_coef[1] + data$INCOME4*two_coef[2] + data$INCOME5*two_coef[3]+ data$HSIZE4*two_coef[4] + data$OWN*two_coef[5])
util_three_body <- exp(data$OWN*three_coef[1] + data$NUMADLT*three_coef[2] + data$WORK3*three_coef[3]+ data$POPDEN*three_coef[4]
                       +  data$child_pres*three_coef[5])
util_four_body <- exp(four_coef)


### Probability computation
total_util <- util_zero_body + util_one_body + util_two_body + util_three_body + util_four_body 
prob_zero_body <- (util_zero_body/total_util)
prob_one_body <- (util_one_body/total_util)
prob_two_body <- (util_two_body/total_util)
prob_three_body <- (util_three_body/total_util)
prob_four_body <- (util_four_body/total_util)


### Cumulative probability computation
cum_prob_zero_body <- prob_zero_body
cum_prob_one_body <- cum_prob_zero_body +  prob_one_body
cum_prob_two_body <- cum_prob_one_body +  prob_two_body 
cum_prob_three_body <- cum_prob_two_body +  prob_three_body 
cum_prob_four_body <- cum_prob_three_body +  prob_four_body 


### Predicted choices
rnum <- runif(4262);
rnum[rnum < cum_prob_zero_body] <- 2;
rnum[rnum < cum_prob_one_body] <- 3;
rnum[rnum < cum_prob_two_body] <- 4;
rnum[rnum < cum_prob_three_body] <- 5;
rnum[rnum <= 1] <- 6;


data$pred_body <- rnum - 2;
data$num_body <- data$Index - 1
comp_obs_pred <- list(data$num_body, data$pred_body)

### Observed vs. predicted comparison charts
multhist(comp_obs_pred, xlab="Number of Body Types", ylab="Percent of Households", col = c("blue", "red"),  main = "Body Type Distribution",
         legend.text = c("Observed", "Predicted"), names.arg = c("0", "1", "2", "3", ">=4"), breaks=seq(-0.5,5, by = 1), ylim=c(0,0.5), freq = FALSE) 

print(proc.time() - pst)
####################################################### End of code ###################################################################################