##### Ordered Probit (OP) model for pickup count ######

##### Description #####
# This code estimates and applies an OP model of pickup count on the same data set #

library("utils")   # to use csv read function
library("foreign") # to use spss read function
library("pastecs") # to use descriptive statistics function
library("mlogit")
library("graphics")
library("VGAM")
library("Zelig")   # This package is for actual mlogit function
library("MASS")    # For ordered Probit
library("ordinal") # Also for ordered probit
library("lmtest")  # for likelihood ratio test
library("aod")
library("plotrix")


pst <- proc.time()

##### Read the data #####
data <- read.table("C:/workspace/VFC_Package_Beta/Model_Estimation/pickups.dat",
col.names = c("HOUSEID",
              "TAZ",
              "RAZ",
              "uno",
              "sero",
              "id",
              "CAR1",
              "CAR2",
              "CAR3",
              "VAN1",
              "VAN2",
              "VAN3",
              "SUV1",
              "SUV2",
              "SUV3",
              "PICK1",
              "PICK2",
              "PICK3",
              "MOTOR",
              "DCAR1",
              "DCAR2",
              "DCAR3",
              "DVAN1",
              "DVAN2",
              "DVAN3",
              "DSUV1",
              "DSUV2",
              "DSUV3",
              "DPICK1",
              "DPICK2",
              "DPICK3",
              "DMOTER",
              "PUMA",
              "DISTRICT",
              "DRVRCNT",
              "HH_HISP",
              "HH_RACE",
              "HHBG",
              "HHC_MSA",
              "HHFAMINC",
              "HHMETDIV",
              "HHRELATD",
              "HHRESP",
              "HHSIZE",
              "HHVEHCNT",
              "HOMEGEO",
              "HOMELAT",
              "HOMELONG",
              "HOMEOWN",
              "HOMETYPE",
              "IMPTHOWN",
              "NUMADLT",
              "RAIL",
              "RESP_CNT",
              "TRAVDAY",
              "URBAN",
              "URBANSIZ",
              "URBRUR",
              "WRKCOUNT",
              "HH13WGT",
              "WTHHFIN",
              "HHR_HISP",
              "HHR_RACE",
              "LIF_CYC",
              "CNTTDHH",
              "numpers",
              "numtrips",
              "TRPCNT",
              "VEHCNT",
              "YEAR",
              "POP1",
              "POP2",
              "POP3",
              "POP4",
              "HH1",
              "HH2",
              "HH3",
              "HH4",
              "EMP1",
              "EMP2",
              "EMP3",
              "EMP4",
              "EMP5",
              "INC1",
              "INC2",
              "INC3",
              "INC4",
              "INC5",
              "TOTAREA",
              "OFFAREA",
              "ENROLL",
              "RFLAG",
              "APASS",
              "DU9",
              "DU19",
              "DU29",
              "DU30",
              "SFHH",
              "MFHH",
              "POPCORR",
              "POPINST",
              "POPMINST",
              "WAHEMP",
              "CONSTEMP",
              "PUBOFF",
              "PUBPUB",
              "ASUENROL",
              "K8",
              "G9TO12",
              "NSBEMP",
              "POPDEN",
              "EMPDEN",
              "OWN",
              "URB",
              "RUR",
              "SF",
              "MF",
              "INCOME1",
              "INCOME2",
              "INCOME3",
              "INCOME4",
              "INCOME5",
              "NUMCHILD",
              "chldpres",
              "WORK0",
              "WORK1",
              "WORK2",
              "WORK3",
              "HSIZE1",
              "HSIZE2",
              "HSIZE3",
              "HSIZE4",
              "bk_mi_sm",
              "wk_mi_sm",
              "tpbkwkin",
              "perbkind",
              "perwkind",
              "AN_MI_in",
              "ANMTR_mi",
              "SF_Own",
              "Retired",
              "popdn100",
              "empdn100",
              "TOT_POP",
              "TOT_EMP",
              "TOTAL_HH",
              "ProHHIN5",
              "ProHHIN1",
              "ProSFHH",
              "ProMFHH",
              "ANNULPB",
              "ANNULRAW",
              "trip_ind",
              "per_ind",
              "nmotr_mi",
              "nmtrminw",
              "test",
              "TRANSIT2",
              "TRANSIT5",
              "AUTO20",
              "AUTO50",
              "RT_TRAN2",
              "RT_TRAN5",
              "RT_AUTO2",
              "RT_AUTO5",
              "CURRENT",
              "FUTURE",
              "Dev_area",
              "TAZ_den",
              "TRANSIT1",
              "TRANSIT3",
              "AUTO10",
              "AUTO30",
              "RT_TRAN1",
              "RT_TRAN3",
              "RT_AUTO1",
              "RT_AUTO3",
              "TZ_dn_Q4",
              "TZ_dn_Q3",
              "TZ_dn_Q2",
              "TZ_dn_Q1",
              "AT10_Q4",
              "AT10_Q3",
              "AT10_Q2",
              "AT10_Q1",
              "AT30_Q4",
              "AT30_Q3",
              "AT30_Q2",
              "AT30_Q1",
              "CAR0_3",
              "CAR4_9",
              "CAR_10",
              "VAN0_3",
              "VAN4_9",
              "VAN_10",
              "SUV0_3",
              "SUV4_9",
              "SUV_10",
              "PICK0_3",
              "PICK4_9",
              "PICK_10",
              "TZDNQ4OD",
              "TZDNQ3OD",
              "TZDNQ2OD",
              "TZDNQ1OD",
              "AT10Q4OD",
              "AT10Q3OD",
              "AT10Q2OD",
              "AT10Q1OD",
              "AT30Q4OD",
              "AT30Q3OD",
              "AT30Q2OD",
              "AT30Q1OD",
              "carmile",
              "vanmile",
              "suvmile",
              "pickmile",
              "NCAR",
              "NVAN",
              "NSUV",
              "NPICK",
              "YCAR0_5",
              "YCAR6_11",
              "YCAR12",
              "YVAN0_5",
              "YVAN6_11",
              "YVAN12",
              "YSUV0_5",
              "YSUV6_11",
              "YSUV12",
              "YPIC0_5",
              "YPIC6_11",
              "YPIC12"),na.strings="-999")


##### Descriptive Statistics #####
# stat.desc(data,basic=TRUE, desc=TRUE, norm=FALSE, p=0.95); 

data$npick1 <- data$NPICK
data$npick1 [data$NPICK >= 3] <- 2
data$npick2 <- data$npick1 - 1
data$obs_npick <- data$npick1
data$npick2 <- factor(data$npick2)
data$pickmile1 <- data$pickmile / 10000

##### Pick-up count, Full Model #####
op_pick_full <- clm(npick2 ~ YPIC0_5 +  YPIC6_11 + pickmile1 + INCOME5 + WORK3 + OWN + empdn100 + TZ_dn_Q3 + AT30_Q1 , data = data, link = 'probit', threshold = 'flexible')
summary(op_pick_full)


##### Constants Only Model #####
op_pick_cons <- clm(npick2 ~ 1, data = data, link = 'probit')
summary(op_pick_cons)


###### Chi Squared test #####
anova(op_pick_full, op_pick_cons, test = "Chisq")


######################################### Application ###############################################
data$const <- 1
coefs <-coef(op_pick_full)
data$cons <- -1*coefs[1]*data$const
expect_op <- data$cons + data$YPIC0_5*coefs[2] +  data$YPIC6_11*coefs[3] + data$pickmile1*coefs[4] + data$INCOME5*coefs[5] + data$WORK3*coefs[6] + data$OWN*coefs[7] + data$empdn100*coefs[8] + data$TZ_dn_Q3*coefs[9] + data$AT30_Q1*coefs[10]

data$p1 <- pnorm(0-expect_op)
data$p2 <- data$const

data$rnum <- runif(length(data$cons))


x = 1:length(data$cons)
for (i in x)
{
  if(data$rnum[i] < data$p1[i]){
    data$pred_npick[i] <- 1
  }
    else{
      data$pred_npick[i] <- 2 
    }
  }


comp_obs_pred <- list(data$obs_npick, data$pred_npick)


### Observed vs. Predicted comparison charts
multhist(comp_obs_pred, xlab="Number of Pick-ups", ylab="Percent of Households", col = c("blue", "red"),  main = "Pick-up Count (N = 1219)",
         legend.text = c("Observed", "Predicted"), breaks=seq(0.5,3, by = 1), , ylim=c(0,1),  freq = FALSE) 

print(proc.time() - pst)
####################################################### End of code ###################################################################################